Package: mariner
Type: Package
Title: Mariner: Explore the Hi-Cs
Version: 1.13.0
Authors@R: c(
    person("Eric", "Davis", role = c("aut", "cre"),
      email = "ericscottdavis@outlook.com",
      comment = c(ORCID = "0000-0003-4051-3217")),
    person("Sarah", "Parker", role = c("aut"),
      email = "smp3800@gmail.com",
      comment = c(ORCID = "0000-0002-2700-3979"))
  )
Description: Tools for manipulating paired ranges and
    working with Hi-C data in R. Functionality includes
    manipulating/merging paired regions, generating
    paired ranges, extracting/aggregating interactions
    from `.hic` files, and visualizing the results. 
    Designed for compatibility with plotgardener for
    visualization.
RoxygenNote: 7.3.3
Depends: R (>= 4.2.0)
Suggests: knitr, testthat (>= 3.0.0), rmarkdown, ExperimentHub,
        marinerData, TxDb.Hsapiens.UCSC.hg38.knownGene, fields
Imports: methods, S4Vectors, BiocGenerics, BiocManager, GenomicRanges,
        InteractionSet, data.table, stats, rlang, glue, assertthat,
        dplyr, magrittr, dbscan, purrr, progress, GenomeInfoDb, strawr
        (>= 0.0.91), DelayedArray, HDF5Array, abind, BiocParallel,
        IRanges, SummarizedExperiment, rhdf5, plotgardener,
        RColorBrewer, colourvalues, utils, grDevices, graphics, grid
biocViews: FunctionalGenomics, Visualization, HiC
VignetteBuilder: knitr
License: MIT + file LICENSE
Encoding: UTF-8
Config/testthat/edition: 3
Collate: 'AllClasses.R' 'AllGenerics.R' 'mariner.R'
        'methods-CountMatrix.R' 'methods-GInteractions.R' 'utils.R'
        'methods-InteractionArray.R' 'methods-InteractionJaggedArray.R'
        'methods-InteractionMatrix.R' 'methods-JaggedArray.R'
        'methods-MatrixSelection.R' 'methods-MergedGInteractions.R'
        'methods-adjustEnrichment.R' 'methods-aggHicMatrices.R'
        'methods-as_ginteractions.R' 'methods-assignToBins.R'
        'methods-binRanges.R' 'methods-calcLoopEnrichment.R'
        'methods-changePixelRes.R' 'methods-hdf5BlockApply.R'
        'methods-makeRandomGInteractions.R' 'methods-mergePairs.R'
        'methods-pileupBoundaries.R' 'methods-pileupDomains.R'
        'methods-pileupPixels.R' 'methods-pixelsToMatrices.R'
        'methods-plotBullseye.R' 'methods-plotMatrix.R'
        'methods-pullHic.R' 'methods-regularize.R'
        'methods-removeShortPairs.R' 'methods-shiftRanges.R'
        'methods-snapToBins.R' 'zzz.R'
URL: https://ericscottdavis.com/mariner/,
        https://github.com/EricSDavis/mariner
git_url: https://git.bioconductor.org/packages/mariner
git_branch: devel
git_last_commit: 9d72228
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-29
NeedsCompilation: no
Packaged: 2026-04-29 22:41:38 UTC; biocbuild
Author: Eric Davis [aut, cre] (ORCID: <https://orcid.org/0000-0003-4051-3217>),
  Sarah Parker [aut] (ORCID: <https://orcid.org/0000-0002-2700-3979>)
Maintainer: Eric Davis <ericscottdavis@outlook.com>
