# Generated by roxygen2: do not edit by hand

S3method(names,MatrixSelection)
export(InteractionArray)
export(InteractionJaggedArray)
export(InteractionMatrix)
export(JaggedArray)
export(MatrixSelection)
export(MergedGInteractions)
export(adjustEnrichment)
export(aggHicMatrices)
export(aggMetadata)
export(as_ginteractions)
export(assignToBins)
export(binRanges)
export(calcLoopEnrichment)
export(changePixelRes)
export(clusters)
export(defaultBuffer)
export(end1)
export(end2)
export(hdf5BlockApply)
export(makeGInteractionsFromDataFrame)
export(makeRandomGInteractions)
export(makeRandomGRanges)
export(mergePairs)
export(pileupBoundaries)
export(pileupDomains)
export(pileupPixels)
export(pixelsToMatrices)
export(plotEnrichment)
export(plotMatrix)
export(pullHicMatrices)
export(pullHicPixels)
export(regularize)
export(removeShortPairs)
export(selectBlock)
export(selectBottomLeft)
export(selectBottomRight)
export(selectCenterPixel)
export(selectCols)
export(selectCoordinates)
export(selectCorners)
export(selectInner)
export(selectOuter)
export(selectPixel)
export(selectRadius)
export(selectRows)
export(selectSubmatrix)
export(selectTopLeft)
export(selectTopRight)
export(selectionMethod)
export(seqnames1)
export(seqnames2)
export(sets)
export(shiftRanges)
export(snapToBins)
export(sources)
export(start1)
export(start2)
exportClasses(DelegatingGInteractions)
exportClasses(InteractionArray)
exportClasses(InteractionJaggedArray)
exportClasses(InteractionMatrix)
exportClasses(JaggedArray)
exportClasses(MatrixSelection)
exportClasses(MergedGInteractions)
exportMethods("$")
exportMethods("+")
exportMethods("-")
exportMethods("[")
exportMethods("[[")
exportMethods("counts<-")
exportMethods("path<-")
exportMethods(InteractionArray)
exportMethods(InteractionMatrix)
exportMethods(adjustEnrichment)
exportMethods(aggHicMatrices)
exportMethods(aggMetadata)
exportMethods(as.list)
exportMethods(as_ginteractions)
exportMethods(assignToBins)
exportMethods(binRanges)
exportMethods(calcLoopEnrichment)
exportMethods(cbind)
exportMethods(changePixelRes)
exportMethods(clusters)
exportMethods(colData)
exportMethods(countOverlaps)
exportMethods(counts)
exportMethods(dim)
exportMethods(end1)
exportMethods(end2)
exportMethods(findOverlaps)
exportMethods(hdf5BlockApply)
exportMethods(interactions)
exportMethods(length)
exportMethods(makeGInteractionsFromDataFrame)
exportMethods(makeRandomGInteractions)
exportMethods(makeRandomGRanges)
exportMethods(mergePairs)
exportMethods(metadata)
exportMethods(overlapsAny)
exportMethods(path)
exportMethods(pileupBoundaries)
exportMethods(pileupDomains)
exportMethods(pileupPixels)
exportMethods(pixelsToMatrices)
exportMethods(plotEnrichment)
exportMethods(plotMatrix)
exportMethods(pullHicMatrices)
exportMethods(pullHicPixels)
exportMethods(rbind)
exportMethods(regularize)
exportMethods(removeShortPairs)
exportMethods(selectBlock)
exportMethods(selectBottomLeft)
exportMethods(selectBottomRight)
exportMethods(selectCenterPixel)
exportMethods(selectCols)
exportMethods(selectCoordinates)
exportMethods(selectCorners)
exportMethods(selectInner)
exportMethods(selectOuter)
exportMethods(selectPixel)
exportMethods(selectRadius)
exportMethods(selectRows)
exportMethods(selectSubmatrix)
exportMethods(selectTopLeft)
exportMethods(selectTopRight)
exportMethods(selectionMethod)
exportMethods(seqnames1)
exportMethods(seqnames2)
exportMethods(sets)
exportMethods(shiftRanges)
exportMethods(show)
exportMethods(snapToBins)
exportMethods(sources)
exportMethods(start1)
exportMethods(start2)
exportMethods(subsetByOverlaps)
import(data.table,
 except = c(between, shift, first, second, indices))
import(methods)
importClassesFrom(DelayedArray,DelayedMatrix)
importClassesFrom(GenomicRanges,GRanges)
importClassesFrom(InteractionSet,GInteractions)
importClassesFrom(InteractionSet,InteractionSet)
importClassesFrom(S4Vectors,DFrame)
importClassesFrom(S4Vectors,DataFrame)
importClassesFrom(S4Vectors,List)
importClassesFrom(S4Vectors,SimpleList)
importClassesFrom(data.table,data.table)
importFrom(BiocGenerics,"counts<-")
importFrom(BiocGenerics,cbind)
importFrom(BiocGenerics,counts)
importFrom(BiocGenerics,rbind)
importFrom(BiocManager,install)
importFrom(BiocParallel,bpparam)
importFrom(DelayedArray,DelayedArray)
importFrom(DelayedArray,RegularArrayGrid)
importFrom(DelayedArray,blockApply)
importFrom(DelayedArray,close)
importFrom(DelayedArray,read_block)
importFrom(DelayedArray,write_block)
importFrom(GenomeInfoDb,seqinfo)
importFrom(GenomeInfoDb,seqlengths)
importFrom(GenomeInfoDb,seqnames)
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,end)
importFrom(GenomicRanges,makeGRangesFromDataFrame)
importFrom(GenomicRanges,resize)
importFrom(GenomicRanges,seqnames)
importFrom(GenomicRanges,shift)
importFrom(GenomicRanges,start)
importFrom(GenomicRanges,strand)
importFrom(GenomicRanges,trim)
importFrom(GenomicRanges,width)
importFrom(HDF5Array,HDF5Array)
importFrom(HDF5Array,HDF5RealizationSink)
importFrom(IRanges,IRanges)
importFrom(IRanges,overlapsAny)
importFrom(IRanges,resize)
importFrom(IRanges,width)
importFrom(InteractionSet,GInteractions)
importFrom(InteractionSet,InteractionSet)
importFrom(InteractionSet,anchorIds)
importFrom(InteractionSet,anchors)
importFrom(InteractionSet,interactions)
importFrom(InteractionSet,intrachr)
importFrom(InteractionSet,pairdist)
importFrom(InteractionSet,reduceRegions)
importFrom(InteractionSet,regions)
importFrom(RColorBrewer,brewer.pal)
importFrom(S4Vectors,"mcols<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,`mcols<-`)
importFrom(S4Vectors,`metadata<-`)
importFrom(S4Vectors,countQueryHits)
importFrom(S4Vectors,elementMetadata)
importFrom(S4Vectors,first)
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,metadata)
importFrom(S4Vectors,parallel_slot_names)
importFrom(S4Vectors,second)
importFrom(S4Vectors,width)
importFrom(SummarizedExperiment,"assays<-")
importFrom(SummarizedExperiment,`colData<-`)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,seqnames)
importFrom(abind,abind)
importFrom(assertthat,is.flag)
importFrom(assertthat,is.number)
importFrom(assertthat,is.string)
importFrom(colourvalues,color_values)
importFrom(data.table,.GRP)
importFrom(data.table,.I)
importFrom(data.table,.SD)
importFrom(data.table,CJ)
importFrom(data.table,`:=`)
importFrom(data.table,as.data.table)
importFrom(data.table,copy)
importFrom(data.table,data.table)
importFrom(data.table,frollapply)
importFrom(data.table,setkeyv)
importFrom(data.table,uniqueN)
importFrom(dbscan,dbscan)
importFrom(dplyr,mutate)
importFrom(glue,double_quote)
importFrom(glue,glue)
importFrom(glue,glue_collapse)
importFrom(glue,single_quote)
importFrom(grDevices,colorRamp)
importFrom(grDevices,colorRampPalette)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(grid,addGrob)
importFrom(grid,gTree)
importFrom(grid,grid.draw)
importFrom(grid,grid.ls)
importFrom(grid,grid.newpage)
importFrom(grid,rasterGrob)
importFrom(grid,unit)
importFrom(grid,viewport)
importFrom(magrittr,`%<>%`)
importFrom(magrittr,`%>%`)
importFrom(plotgardener,mapColors)
importFrom(progress,progress_bar)
importFrom(purrr,pmap)
importFrom(rhdf5,h5createDataset)
importFrom(rhdf5,h5createFile)
importFrom(rhdf5,h5read)
importFrom(rhdf5,h5write)
importFrom(rlang,abort)
importFrom(rlang,arg_match)
importFrom(rlang,inform)
importFrom(rlang,parse_expr)
importFrom(rlang,warn)
importFrom(stats,dist)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,predict)
importFrom(stats,setNames)
importFrom(stats,smooth.spline)
importFrom(strawr,readHicBpResolutions)
importFrom(strawr,readHicChroms)
importFrom(strawr,readHicNormTypes)
importFrom(strawr,straw)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,tail)
importMethodsFrom(IRanges,countOverlaps)
importMethodsFrom(IRanges,findOverlaps)
importMethodsFrom(IRanges,overlapsAny)
importMethodsFrom(IRanges,subsetByOverlaps)
importMethodsFrom(S4Vectors,as.list)
